﻿'use strict';

var callDashboard = {
    sortableRemove: function (event, ui) {
        if ($(this).closest(".callboard-items").find(".callboard-item").length > 0) {
            var items = $(this).closest(".callboard-layout-column").find(".callboard-id").map(function () {
                return $(this).val();
            }).get();
            $.ajax({
                url: RQ_URL + '/Call/ChangeDashboardItemsPositions',
                data: {
                    items: items,
                    layoutPosition: $(this).closest(".callboard-layout-column").index()
                },
                type: "POST"
            });
        }
    },
    sortableStop: function (event, ui) {
        var items = $(ui.item).closest(".callboard-layout-column").find(".callboard-id").map(function () {
            return $(this).val();
        }).get();
        $.ajax({
            url: RQ_URL + '/Call/ChangeDashboardItemsPositions',
            data: {
                items: items,
                layoutPosition: $(ui.item).closest(".callboard-layout-column").index()
            },
            type: "POST"
        });
    }
};

$(function () {
    var viewModel;
    function subscribeViewModel(newViewModel) {
        viewModel = newViewModel;

        navigator.serviceWorker.addEventListener('message', function (event) {
            switch (event.data.action) {
                case "open":
                    newViewModel.selectCallItem(event.data.uniqueId);
                    break;
            }
        });
    }

    function getSubscribedViewModel() {
        return viewModel;
    }

    function initProcessAllCallItemsConfirmDialog() {
        $("#processAllCallItemsConfirmDialog").dialog({
            autoOpen: false,
            modal: true,
            resizable: false,
            width: 'auto',
            height: 'auto',
            buttons: {
                "Ano": function () {
                    $(this).dialog("close");
                    OnAjaxBegin();
                    viewModel.processCallItems().then(function () {
                        OnAjaxComplete();
                    }).catch(function () {
                        OnAjaxComplete();
                    });
                },
                "Ne": function () {
                    $(this).dialog("close");
                }
            }
        });
    }

    function initCallTicketRelationDialog() {
        $("#dialog-callTicketRelation").dialog({
            autoOpen: false,
            resizable: false,
            width: "900",
            height: "auto",
            modal: true,
            buttons: [
                {
                    text: Localization_Save,
                    click: function () {
                        var selectedCallItem = viewModel.selectedCallItem();
                        if (selectedCallItem) {
                            selectedCallItem.createTicketRelations($("#CallSessionTicket").val()).then(function () {
                                $("#dialog-callTicketRelation").dialog("close");
                            });
                        }
                    }
                },
                {
                    text: Localization_Close,
                    click: function () {
                        $(this).dialog("close");
                    }
                }
            ]
        });
    }

    function initCallUserRelationDialog() {
        $("#dialog-callUserRelation").dialog({
            autoOpen: false,
            resizable: false,
            width: "500",
            height: "auto",
            modal: true,
            buttons: [
                {
                    text: Localization_Save,
                    click: function () {

                        var userProviderKey = $('#ddlCallUser').val();

                        if (userProviderKey === null || !userProviderKey.trim()) {
                            $('#lbErrorNoUser').show();
                        }
                        else {
                            $('#lbErrorNoUser').hide();

                            var selectedCallItem = viewModel.selectedCallItem();
                            if (selectedCallItem)
                            {
                                selectedCallItem.setUserOfCallItem(userProviderKey).then(function () {
                                    $("#dialog-callUserRelation").dialog("close");
                                });
                            }
                        }
                    }
                },
                {
                    text: Localization_Close,
                    click: function () {
                        $(this).dialog("close");
                    }
                }
            ]
        });

        $("#ddlCallUser").select2({
            width: '100%',
            minimumInputLength: 0,
            placeholder: Localization_Search,
            ajax: {
                url: RQ_URL + '/Account/UsersForSelect2Ajax',
                type: "POST",
                dataType: 'json',
                delay: 250,
                data: function (params) {
                    return {
                        pageSize: 10,
                        pageNum: params.page || 1,
                        term: params.term,
                        showCustomerName: true,
                        showEmail: true,
                        searchInEmail: true,
                        searchInCustomer: true
                    };
                },
                processResults: function (data, params) {
                    params.page = data.PageNum;

                    return {
                        results: data.Results,
                        pagination: {
                            more: data.PageNum * data.PageSize < data.Total
                        }
                    };
                },
                cache: true
            },
            escapeMarkup: function (markup) { return markup; },
            templateResult: formatUsersForSelect2AjaxData,
            templateSelection: formatUsersForSelect2AjaxElement
        });
    }

    function AddUser() {
        $("#dialogAddNewUser").html("<div class='loading-32'></div>");

        $("#dialogAddNewUser").dialog({
            autoOpen: true,
            resizable: false,
            width: "auto",
            height: "auto",
            modal: true
        });

        var selectedCallItem = viewModel.selectedCallItem();
        $("#dialogAddNewUser").load(RQ_URL + "/Account/RegisterUserDialogPartial", {
            phoneNumber: selectedCallItem && (selectedCallItem.way === "In" ? selectedCallItem.from : selectedCallItem.way === "Out" ? selectedCallItem.to : null)
        }, function () {
            Select2Combos();
            $("#dialogAddNewUser").dialog("close");

            $("#dialogAddNewUser").dialog({
                autoOpen: true,
                resizable: false,
                width: 850,
                height: "auto",
                modal: true,
                buttons: [
                    {
                        text: Localization_Create,
                        click: function () {
                            $('#formRegisterUser').submit();
                        }
                    },
                    {
                        text: Localization_Cancel,
                        click: function () {
                            $(this).dialog("close");
                        }
                    }
                ]
            });
        });
    }

    function initSelect2TreeHandler() {
        ko.bindingHandlers.select2tree = {
            init: function (element, valueAccessor, allBindingsAccessor) {
                $(element).select2tree({
                    width: '100%',
                    placeholder: ko.utils.unwrapObservable(valueAccessor())
                });
                ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                    $(element).select2tree('destroy');
                });
            }
        };
    }

    function initSortableHandler() {
        ko.bindingHandlers.sortable = {
            init: function (element, valueAccessor, allBindingsAccessor) {
                $(element).sortable(valueAccessor());

                ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                    $(element).sortable("destroy");
                });
            }
        };
    }

    function openCallTicketRelation() {
        $("#CallSessionTicket").val([]).trigger('change');
        $("#dialog-callTicketRelation").html("<div class='loading-32'></div>");
        $("#dialog-callTicketRelation").dialog("open");
        $("#dialog-callTicketRelation").load(RQ_URL + "/Call/CallTicketRelation/" + viewModel.selectedCallItemUniqueId(), function () {
            $("#dialog-callTicketRelation").dialog("close");
            $("#dialog-callTicketRelation").dialog("open");
        });
    }

    function initDOMEvents() {
        $('.bt-add-new-user').click(function () {
            AddUser();
        });
    }

    initDOMEvents();
    initSelect2TreeHandler();
    initSortableHandler();
    initProcessAllCallItemsConfirmDialog();
    initCallTicketRelationDialog();
    initCallUserRelationDialog();
    callDashboard.subscribeViewModel = subscribeViewModel;
    callDashboard.getSubscribedViewModel = getSubscribedViewModel;
    callDashboard.openCallTicketRelation = openCallTicketRelation;

});

// globalni funkce, je volana z dialogu pro vytvoreni noveho uzivatele
function OnRegisterUserSuccess(result) {
    if (result.success) {
        $("#dialogAddNewUser").dialog("close");
        $("#dialogAddNewUserComplete").html(result.html); //continue to next form
        $("#dialogAddNewUserComplete").dialog({
            autoOpen: true,
            resizable: false,
            width: 550,
            height: "auto",
            modal: true,
            buttons: [
                {
                    text: Localization_Select,
                    click: function () {
                        $(this).dialog("close");

                        var viewModel = callDashboard.getSubscribedViewModel();
                        var selectedCallItem = viewModel.selectedCallItem();
                        if (selectedCallItem) {
                            selectedCallItem.setUserOfCallItem(result.user.id).then(function () {
                                $("#dialog-callUserRelation").dialog("close");
                            });
                        }
                    }
                }
            ]
        });
    }
    else {
        $('#dialogAddNewUser').html(result); //some form errors, redisplay form
    }
}